#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from typing import Dict, List, Optional, Union

import tripayiin

import tripayiin.exception
from tripayiin.types.code import Code
from tripayiin.types.orderItem import OrderItem
from tripayiin.types.response import Response


class CreatePayment:
    async def createClosedPayment(
        self: "tripayiin.Tripayiin",
        method: Code,
        merchantRef: str,
        amount: int,
        customerName: str,
        customerEmail: str,
        orderItems: List[OrderItem],
        merchantCode: Optional[str] = None,
        customerPhone: Optional[str] = None,
        callbackUrl: Optional[str] = None,
        returnUrl: Optional[str] = None,
        expiredTime: Optional[int] = None
    ) -> Union[Union[Dict, List[Dict]], Response]:
        """
        Create Closed Payment

        Args:
            method (`Code`): Payment channel code, see [Payment Channel Code](https://tripay.co.id/developer?tab=channels)
            merchantRef (`str`): Merchant reference
            amount (`int`): Transaction nominal
            customerName (`str`): Customer name
            customerEmail (`str`): Customer email
            orderItems (`List[OrderItem]`): List of order items
            merchantCode (`str`, optional): Merchant code. Defaults to None.
            customerPhone (`str`, optional): Customer phone number. Defaults to None.
            callbackUrl (`str`, optional): Callback URL. Defaults to None.
            returnUrl (`str`, optional): Return URL. Defaults to None.
            expiredTime (`int`, optional): Expired time in timestamp. Defaults to None.

        Returns:
            Dict: Returns a dictionary
        """
        data = {
            "method": method,
            "merchant_ref": merchantRef,
            "amount": amount,
            "customer_name": customerName,
            "customer_email": customerEmail,
            "order_items": orderItems,
        }
        if customerPhone is not None:
            data["customer_phone"] = customerPhone
        if callbackUrl is not None:
            data["callback_url"] = callbackUrl
        if returnUrl is not None:
            data["return_url"] = returnUrl
        if expiredTime is not None:
            data["expired_time"] = expiredTime
        
        if self._merchantCode is None and merchantCode is None:
            raise tripayiin.exception.MerchantCodeError("You must set Merchant Code with `setMerchantCode` method or with this parameter `merchantCode`")
        signature = await self.createSignature(
            merchantCode=self._merchantCode if self._merchantCode is not None else merchantCode,
            merchantRef=merchantRef,
            amount=str(amount)
        )
        data["signature"] = signature
        res = await self.post(
            url="/transaction/create",
            data=data
        )
        if res.data:
            return res.data
        return res



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from typing import Union


class Response:
    def __init__(self, **kwargs):
        self.success: bool = kwargs.get("success", False)
        self.message: str = kwargs.get("message", "")
        self.data: Union[Union[dict, list], None] = kwargs.get("data", None)

    def __repr__(self):
        return f"Response(success={self.success}, message={self.message}, data={self.data})"

    def parse(self):
        return {
            "success": self.success,
            "message": self.message,
            "data": self.data
        }



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from enum import Enum


class Code(str, Enum):
    MYBVA = "MYBVA"
    QRIS_SHOPEEPAY = "QRIS_SHOPEEPAY"
    SHOPEEPAY = "SHOPEEPAY"
    DANA = "DANA"
    QRIS2 = "QRIS2"
    QRISC = "QRISC"
    QRIS = "QRIS"
    OVO = "OVO"
    ALFAMIDI = "ALFAMIDI"
    INDOMARET = "INDOMARET"
    ALFAMART = "ALFAMART"
    OTHERBANKVA = "OTHERBANKVA"
    DANAMONVA = "DANAMONVA"
    OCBCVA = "OCBCVA"
    BSIVA = "BSIVA"
    CIMBVA = "CIMBVA"
    MUAMALATVA = "MUAMALATVA"
    BCAVA = "BCAVA"
    MANDIRIVA = "MANDIRIVA"
    BRIVA = "BRIVA"
    BNIVA = "BNIVA"
    PERMATAVA = "PERMATAVA"



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from typing import Dict, List, Optional

import tripayiin


class CostCalculator:
    async def costCalculator(
        self: "tripayiin.Tripayiin",
        amount: int,
        method: Optional[str] = None
    ) -> List[Dict]:
        """
        Get Cost Calculator

        Args:
            amount (`int`): Transaction nominal
            method (`str`, optional): Payment channel code. If this parameter is empty then the result is a list of all available payment channels. Defaults to None.

        Returns:
            List[Dict]: Returns a list of dictionaries
        """
        data = {"amount": amount}
        if method is not None:
            data["method"] = method
        res = await self.get(
            url="/merchant/fee-calculator",
            data=data
        )
        return res['data']



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from typing import Dict, List, Optional

import tripayiin
from tripayiin.types.code import Code


class TransactionList:
    async def transactionList(
        self: "tripayiin.Tripayiin",
        page: Optional[int] = None,
        perPage: Optional[int] = None,
        sort: Optional[str] = None,
        references: Optional[str] = None,
        merchantRef: Optional[str] = None,
        method: Optional[Code] = None,
        status: Optional[str] = None
    ) -> List[Dict]:
        """
        Get Cost Calculator

        Args:
            amount (`int`): Transaction nominal
            method (`str`, optional): Payment channel code. If this parameter is empty then the result is a list of all available payment channels. Defaults to None.
            

        Returns:
            Dict[str, Any]: Returns a dictionaries
        """
        data = {}
        if method is not None:
            data["method"] = method
        if status is not None:
            data["status"] = status
        if page is not None:
            data["page"] = page
        if perPage is not None:
            data["per_page"] = perPage
        if sort is not None:
            data["sort"] = sort
        if references is not None:
            data["references"] = references
        if merchantRef is not None:
            data["merchant_ref"] = merchantRef

        res = await self.get(
            url="/merchant/transactions",
            data=data
        )
        return {
            "data": res['data'],
            "pagination": res['pagination']
        }



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

import random
from datetime import datetime
from pytz import timezone
from typing import Literal, Optional, Union
from babel.numbers import format_currency, NumberPattern

from tripayiin.api import Api
from tripayiin.method import Method


__version__ = "1.0.2"


class Tripayiin(Api, Method):
    version: str = __version__
    def __init__(
        self,
        apiKey: str,
        privateKey: str,
        sandbox: Optional[bool],
        merchantCode: Optional[str] = None
    ):
        super(Tripayiin, self).__init__(
            apiKey=apiKey,
            privateKey=privateKey,
            sandbox=sandbox
        )
        self._sandbox = sandbox if sandbox is not None else False
        self._apiKey = apiKey
        self._privateKey = privateKey
        self._merchantCode = merchantCode

    def setMerchantCode(self, merchantCode: str):
        self._merchantCode = merchantCode

    def formatCurrency(
        self,
        price: str,
        currency: str = "IDR",
        format: Optional[Union[str, NumberPattern]] = None,
        locale: Optional[str] = "id_ID",
        currency_digits: bool = True,
        format_type: Literal['name', 'standard', 'accounting'] = "standard",
        decimal_quantization: bool = True,
        group_separator: bool = True,
        *,
        numbering_system: Union[str, Literal['default']] = "latn"
    ):
        currency = format_currency(
            number=price,
            currency=currency,
            locale=locale,
            currency_digits=currency_digits,
            format=format,
            format_type=format_type,
            decimal_quantization=decimal_quantization,
            group_separator=group_separator,
            numbering_system=numbering_system
        )
        return currency

    def genUniqCode(self, name: str, length: int):
        _string = '1 2 3 4 5 6 7 8 9 0'.split()
        unikCode: str = name
        for x in range(length):
            unikCode += random.choice(_string)
        return unikCode

    def reformatDate(
        self,
        timestamp: Union[int, float],
        format: str = "%d-%m-%Y %H:%M:%S",
        zone: Optional[str] = "Asia/Jakarta"
    ):
        # Format TimeStamp To Date
        date = datetime.fromtimestamp(timestamp, tz=timezone(zone))
        return date.strftime(format)



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from .merchant import Merchant
from .payment import Payment
from .transaction import Transaction


class Method(
    Merchant,
    Payment,
    Transaction
):
    pass



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from typing import Optional, Union


class OrderItem:
    def __init__(
        self,
        sku: str,
        productName: str,
        price: Union[int, float],
        quantity: int,
        productUrl: Optional[str] = None,
        imageUrl: Optional[str] = None
    ):
        self.sku = sku
        self.name = productName
        self.price = price
        self.quantity = quantity
        self.productUrl = productUrl if productUrl is not None else ''
        self.imageUrl = imageUrl if imageUrl is not None else ''

    def __repr__(self):
        return f"OrderItem(sku={self.sku}, name={self.name}, price={self.price}, quantity={self.quantity}, product_url={self.productUrl}, image_url={self.imageUrl})"

    def parse(self):
        return {
            "sku": self.sku,
            "name": self.name,
            "price": self.price,
            "quantity": self.quantity,
            "product_url": self.productUrl,
            "image_url": self.imageUrl
        }



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from typing import Dict, List, Optional, Union

import tripayiin
from tripayiin.types.code import Code


class Intruction:
    async def intructionPayment(
        self: "tripayiin.Tripayiin",
        code: Code,
        payCode: Optional[str] = None,
        amount: Optional[int] = None,
        allowHtml: Optional[int] = None
    ) -> Union[List[Dict], None]:
        """
        Get Payment Instruction

        Args:
            code (`Code`): Payment Channel Code, see [Payment Channel Code](https://tripay.co.id/developer?tab=channels)
            payCode (`str`, optional): To enter the payment code/VA number into the instruction response. Defaults to None.
            amount (`int`, optional): To enter a nominal value into the instruction response. Defaults to None.
            allowHtml (`int`, optional): To allow html tags on the instruction
            0 = Not allowed
            1 = Allowed.
            Defaults to 1.

        Returns:
            Union[List[Dict], None]: Returns a list of dictionaries if successful, otherwise None
        """
        data = {
            "code": code
        }
        if payCode is not None:
            data["pay_code"] = payCode
        if amount is not None:
            data["amount"] = amount
        if allowHtml is not None:
            data["allow_html"] = allowHtml
        res = await self.get(
            url="/payment/instruction",
            data=data
        )
        if res['data'] is None:
            return None
        return res['data']



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.


from aiohttp import ClientSession
import hmac
import hashlib

from typing import Optional
from tripayiin.types.response import Response


class Api:
    sandboxUrl: str = "https://tripay.co.id/api-sandbox"
    productionUrl: str = "https://tripay.co.id/api"
    def __init__(self, apiKey: str, privateKey: str, sandbox: Optional[bool]):
        self._apiKey = apiKey
        self._privateKey = privateKey
        self.baseUrl = self.sandboxUrl if sandbox is True else self.productionUrl

    async def get(self, url: str, data: Optional[dict] = None, headers: Optional[dict] = None) -> Response:
        async with ClientSession(
            headers={
                "Authorization": f"Bearer {self._apiKey}",
                "Content-Type": "application/json"
            }
        ) as response:
            
            res = await response.get(
                url=f"{self.baseUrl}{url}",
                json=data,
                headers=headers
            )
            json = await res.json()
            return json

    async def post(self, url: str, data: Optional[dict] = None, headers: Optional[dict] = None) -> Response:
        async with ClientSession(
            headers={
                "Authorization": f"Bearer {self._apiKey}",
                "Content-Type": "application/json"
            }
        ) as response:
            res = await response.post(
                url=f"{self.baseUrl}{url}",
                json=data,
                headers=headers
            )
            json = await res.json()
            return json

    async def createSignature(
        self,
        merchantCode: str,
        merchantRef: Optional[str] = None,
        amount: Optional[str] = None,
        channel: Optional[str] = None
    ):
        signStr = f"{merchantCode}{merchantRef if merchantRef is not None else channel}{amount if amount is not None else merchantRef}"
        signature = hmac.new(bytes(self._privateKey,'latin-1'), bytes(signStr,'latin-1'), hashlib.sha256).hexdigest()
        return signature



#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import shutil
import sys
from setuptools import setup, find_packages

def clearFolder(folder):
    try:
        # Remove Directory
        if os.path.exists(folder):
            shutil.rmtree(folder)
    except Exception as e:
        print(e)


with open("tripayiin/__init__.py", encoding="utf-8") as f:
    version = re.findall(r"__version__ = \"(.+)\"", f.read())[0]

with open("README.md", encoding="utf-8") as f:
    readme = f.read()

if sys.argv[-1] == "publish":
    clearFolder("build")
    clearFolder("dist")
    clearFolder("tripayiin.egg-info")
    os.system("pip install twine setuptools")
    os.system("python setup.py sdist")
    os.system("twine upload dist/*")
    sys.exit()

setup(
    name="tripayiin",
    version=version,
    description="TriPay Payment Gateaway Client For Python (Unofficial)",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/AyiinXd",
    download_url="https://github.com/AyiinXd/tripayiin/releases/latest",
    author="AyiinXd",
    author_email="ayiincorporation@gmail.com",
    license="LGPLv3",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Utilities"
    ],
    keywords="tripay payment gateway",
    project_urls={
        "Tracker": "https://github.com/AyiinXd/tripayiin/issues",
        "Community": "https://t.me/AyiinChats",
        "Source": "https://github.com/AyiinXd/tripayiin",
    },
    python_requires="~=3.8",
    package_data={
        "tripayiin": ["py.typed"],
    },
    packages=find_packages(exclude=["tests*"]),
    zip_safe=False,
    install_requires=[
        "aiohttp",
        "babel",
        "pytz"
    ],
)


#  TriPayiin - Payment Gateaway Client For Python (Unofficial)
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of TriPayiin.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TriPayiin.  If not, see <http://www.gnu.org/licenses/>.

from typing import Dict, List, Optional, Union

import tripayiin

import tripayiin.exception as exception
from tripayiin.types.code import Code
from tripayiin.types.response import Response


class CreatePayment:
    async def createOpenPayment(
        self: "tripayiin.Tripayiin",
        method: Code,
        merchantRef: str,
        customerName: Optional[str] = None,
        merchantCode: Optional[str] = None
    ) -> Union[Union[Dict, List[Dict]], Response]:
        """
        Create Open Payment

        Args:
            method (`Code`): Payment channel code, see [Payment Channel Code](https://tripay.co.id/developer?tab=channels)
            merchantRef (`str`, optional): Merchant reference. Defaults to None.
            customerName (`str`, optional): Customer name. Defaults to None.
            merchantCode (`str`, optional): Merchant code. Defaults to None.

        Returns:
            Dict: Returns a dictionary
        """
        if self._sandbox is True:
            raise exception.SandboxModeError()
        data = {
            "method": method
        }
        if merchantRef is not None:
            data["merchant_ref"] = merchantRef
        if customerName is not None:
            data["customer_name"] = customerName
        
        if self._merchantCode is None and merchantCode is None:
            raise exception.MerchantCodeError("You must set Merchant Code with `setMerchantCode` method or with this parameter `merchantCode`")
        signature = await self.createSignature(
            merchant_code=self._merchantCode if self._merchantCode is not None else merchantCode,
            channel=method,
            merchantRef=merchantRef
        )
        data["signature"] = signature
        res = await self.post(
            url="/open-payment/create",
            data=data
        )
        if res.data:
            return res.data
        else:
            return res